//--------------------------------------------------------------------------------------
// 
// WPF ShaderEffect HLSL -- Color Blindness
// 
//--------------------------------------------------------------------------------------

static const float DeficiencyMultipliers[8][9] =
{
                          // ------------------------------------------------------------------------------------
                          //        RED                         GREEN                         BLUE
                          // ------------------------|----------------------------|------------------------------
                          //  R      G      B        |     R      G      B        |     R      G      B        
                          // ------------------------|----------------------------|------------------------------
     /*    Protanopia */   {  0.567, 0.433, 0,             0.558, 0.442, 0,             0,     0.242, 0.758,   },  
     /*   Protanomaly */   {  0.817, 0.183, 0,             0.333, 0.667, 0,             0,     0.125, 0.875,   }, 
     /*  Deuteranopia */   {  0.625, 0.375, 0,             0.7,   0.3,   0,             0,     0.3,   0.7,     }, 
     /* Deuteranomaly */   {  0.8,   0.2,   0,             0.258, 0.742, 0,             0,     0.142, 0.858,   }, 
     /*    Tritanopia */   {  0.95,  0.05,  0,             0,     0.433, 0.567,         0,     0.475, 0.525,   }, 
     /*   Tritanomaly */   {  0.967, 0.033, 0,             0,     0.733, 0.267,         0,     0.183, 0.817,   }, 
     /* Achromatopsia */   {  0.299, 0.587, 0.114,         0.299, 0.587, 0.114,         0.299, 0.587, 0.114,   }, 
     /* Achromatomaly */   {  0.618, 0.320, 0.062,         0.163, 0.775, 0.062,         0.163, 0.320, 0.516    } 
};


//-----------------------------------------------------------------------------------------
// Shader constant register mappings (scalars - float, double, Point, Color, Point3D, etc.)
//-----------------------------------------------------------------------------------------

float deficiencyIndex : register(C0);

//--------------------------------------------------------------------------------------
// Sampler Inputs (Brushes, including ImplicitInput)
//--------------------------------------------------------------------------------------

sampler2D implicitInput : register(S0);

//--------------------------------------------------------------------------------------
// Pixel Shader
//--------------------------------------------------------------------------------------

float4 main( float2 uv : TEXCOORD0 ) : COLOR
{
    float4 input = tex2D(implicitInput, uv);
    
    if (deficiencyIndex < 0) 
        return input;
    
    float multiplier[] = DeficiencyMultipliers[deficiencyIndex];
    
    float4 color;
    color.r = 0.0019 + input.r * multiplier[0]  + input.g * multiplier[1]  + input.b * multiplier[2] ;
    color.g = 0.0019 + input.r * multiplier[3]  + input.g * multiplier[4]  + input.b * multiplier[5] ;
    color.b = 0.0019 + input.r * multiplier[6]  + input.g * multiplier[7]  + input.b * multiplier[8] ;
    color.a = input.a;
    
    return color;
}